<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xs="http://www.w3.org/2001/XMLSchema" exclude-result-prefixes="xs"
    xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl" version="2.0">
    <xd:doc scope="stylesheet">
        <xd:desc>
            <xd:p><xd:b>Created on:</xd:b> Jun 20, 2011</xd:p>
            <xd:p><xd:b>Author:</xd:b> DRRW</xd:p>
            <xd:p></xd:p>
        </xd:desc>
    </xd:doc>
    
    <xsl:param name="debug">0</xsl:param>
    
    <xsl:variable name="rules" select="/Dictionary/Rules"/>
    
    <xsl:template match="*">
        <xsl:message>Starting</xsl:message>
        <xsl:variable name="testResults">
        <Results>
            <xsl:message> - processing <xsl:value-of select="count(Parent)"/> parent nodes</xsl:message>
            <xsl:message> - total rules <xsl:value-of select="count($rules/Rule)"/></xsl:message>
        <xsl:apply-templates select="$rules/Rule"/>
        <xsl:apply-templates select="Parent"/>
        </Results>
        </xsl:variable>
        <xsl:message> - orphan rules <xsl:value-of select="count($testResults//warn)"/></xsl:message>
        <xsl:choose>
            <xsl:when test="count($testResults//error) = 0"><xsl:message>Dictionary rules passed validation.</xsl:message></xsl:when>
            <xsl:otherwise><xsl:message>ERROR! Total of <xsl:value-of select="count($testResults//error)"/> dictionary rules failed validation.</xsl:message></xsl:otherwise>
        </xsl:choose>
        <xsl:copy-of select="$testResults"/>
        <xsl:message>Finished</xsl:message>
    </xsl:template>
    
    <xsl:template match="Parent">
        <xsl:variable name="uid" select="@UID"/>
        <xsl:if test="@UID and(not($rules//Rule[@UID = $uid][@type]))"><error UID="{@UID}" name="{@name}"/><xsl:message> - missing Parent <xsl:value-of select="if(not($rules//Rule[@UID = $uid])) then 'rule' else 'type'"/> - <xsl:value-of select="@name"/> - <xsl:value-of select="@UID"/></xsl:message></xsl:if>
        <xsl:apply-templates select="Parent"/>
        <xsl:apply-templates select="Child"/>
        <xsl:apply-templates select="Attribute"/>
    </xsl:template>
    
    <xsl:template match="Child">
        <xsl:variable name="uid" select="@UID"/>
        <xsl:if test="@UID and(not($rules//Rule[@UID = $uid][@type]))"><error UID="{@UID}" name="{@name}"/><xsl:message> - missing Child <xsl:value-of select="if(not($rules//Rule[@UID = $uid])) then 'rule' else 'type'"/> - <xsl:value-of select="@name"/> - <xsl:value-of select="@UID"/></xsl:message></xsl:if>
        <xsl:apply-templates select="Attribute"/>
    </xsl:template>
    
    <xsl:template match="Attribute">
        <xsl:variable name="uid" select="@UID"/>
        <xsl:if test="@UID and(not($rules//Rule[@UID = $uid][@type]))"><error UID="{@UID}" name="{@name}"/><xsl:message> - missing Attribute <xsl:value-of select="if(not($rules//Rule[@UID = $uid])) then 'rule' else 'type'"/> - <xsl:value-of select="@name"/> - <xsl:value-of select="@UID"/></xsl:message></xsl:if>
    </xsl:template>
    
    <xsl:template match="Rule">
        <xsl:variable name="uid" select="@UID"/>
        <xsl:choose>
            <xsl:when test="//Parent[@UID = $uid]"/>
            <xsl:when test="//Child[@UID = $uid]"/>
            <xsl:when test="//Attribute[@UID = $uid]"/>
            <xsl:when test="@type = 'token'"/>
            <xsl:otherwise>
                <warn UID="{@UID}" name="rule"/><!-- xsl:message> - orphan rule - <xsl:value-of select="@UID"/></xsl:message -->
            </xsl:otherwise>
        </xsl:choose>
        
    </xsl:template>
    
</xsl:stylesheet>
